
#include <iostream>

#include "ijkscalarfield.txx"

typedef float COORD_TYPE;

const int DIM2(2);
const int DIM3(3);
const int DIM4(4);
const float max_small_magnitude(0.001);

using namespace std;

void test_rot(const int dimension);
void read_vector(const char * prompt, const int dimension, COORD_TYPE v[]);
void write_vector(const char * s, const int dimension, const COORD_TYPE v[],
                  const bool flag_endl);

int main()
{
  int dimension;

  cout << "Enter dimension: ";
  cin >> dimension;

  test_rot(dimension);

  return 0;
}

void read_vector(const char * prompt, const int dimension, COORD_TYPE v[])
{
  cout << prompt;
  for (int d = 0; d < dimension; d++)
    { cin >> v[d]; }
}

void write_vector(const char * s, const int dimension, const COORD_TYPE v[],
                  const bool flag_endl)
{
  cout << s;
  for (int d = 0; d < dimension; d++)
    { cout << " " << v[d]; }
  if (flag_endl) { cout << endl; }
}

void test_rot(const int dimension)
{
  COORD_TYPE v0[dimension], v1[dimension];
  COORD_TYPE w0[dimension], w1[dimension];
  COORD_TYPE mag0, mag1;

  read_vector("Enter start vector: ", dimension, v0);
  read_vector("Enter target vector: ", dimension, v1);
  read_vector("Enter w0: ", dimension, w0);

  IJKSCALARFIELD::rotate_vector
    (dimension, max_small_magnitude, v0, v1, w0, w1);
  write_vector("w1: ", dimension, w1, true);

  IJK::compute_magnitude(dimension, w0, mag0);
  IJK::compute_magnitude(dimension, w1, mag1);

  cout << "magnitude w0: " << mag0 << endl;
  cout << "magnitude w1: " << mag1 << endl;

  if (dimension == DIM3) {
    COORD_TYPE v2[dimension];
    COORD_TYPE v0_orth[dimension], v1_orth[dimension];
    COORD_TYPE u2[dimension];
    COORD_TYPE a0, a1;

    IJK::normalize_vector(dimension, v0, max_small_magnitude, v0);
    IJK::normalize_vector(dimension, v1, max_small_magnitude, v1);
    IJKSCALARFIELD::cross_product3D(v0, v1, v2);
    IJK::normalize_vector(dimension, v2, max_small_magnitude, v2);
    IJKSCALARFIELD::cross_product3D(v2, v0, v0_orth);
    IJKSCALARFIELD::cross_product3D(v2, v1, v1_orth);

    IJK::compute_inner_product(dimension, w0, v0, a0);
    IJK::compute_inner_product(dimension, w0, v0_orth, a1);
    IJK::project_vector(dimension, w0, v2, u2);

    IJK::copy_coord(dimension, u2, w1);
    IJK::add_scaled_coord(dimension, a0, v1, w1, w1);
    IJK::add_scaled_coord(dimension, a1, v1_orth, w1, w1);

    write_vector("w1 (3D transform): ", dimension, w1, true);
  }
}
